% This m-file structurally estimates the transportation costs and search
% probabilities given observed trade flows and prices

% written by Treb on Aug 8, 2013

%% Loading the data
    % cd 'C:\Users\Treb Allen\Dropbox\Research\Trade\Information\Empirics\Structural Estimation - Monthly'
    cd 'C:\Users\dwi902\Dropbox\Research\Trade\Information\Empirics\Structural Estimation - Monthly'
    if (matlabpool('size')) == 0
        matlabpool local 12
    end
    clear
    clc
    
    % getting the data from stata
        stata_id = load('id_082213.dat');
        stata_theta = load('theta_082213.dat');
        stata_trade = load('tradeflows_082213.dat');
        stata_dest_prices = load('dest_prices_082213.dat');
        stata_orig_prices = load('orig_prices_082213.dat');
        stata_production = load('production_082213.dat');
        
    % getting the missing values from Stata
        stata_trade(stata_trade==999) = NaN;
        stata_dest_prices(stata_dest_prices==999) = NaN;
        
    % constructing the matrices
        N_orig = max(stata_id(:,1)); % number of origins
        N_dest = size(stata_dest_prices,2); % number of destinations
        C = max(stata_id(:,2)); % number of commodities
        T = max(stata_id(:,3)); % number of years
        M = max(stata_id(:,4)); % number of months

    % Trade flows
        Q_trade = NaN(N_orig,N_dest,C,T); % origin-destination-commodity-year trade flows
        parfor i=1:N_orig
            for c=1:C
                for t=1:T
                    temp_loc = find(stata_id(:,1)==i & stata_id(:,2)==c & stata_id(:,3)==t);
                    if size(temp_loc,1)>0 % then we have an entry
                        Q_trade(i,:,c,t) = nanmean(stata_trade(temp_loc,:),1);
                    end
                end
            end
        end
        
    % Prices
        price_orig = NaN(N_orig,C,T,M); % origin-commodity-year-month prices
        price_dest = NaN(N_orig,N_dest,C,T,M); % origind-destination-commodity-year-month prices
        parfor i=1:N_orig
            for c=1:C
                for t=1:T
                    for m=1:M
                        temp_loc = find(stata_id(:,1)==i & stata_id(:,2)==c & stata_id(:,3)==t & stata_id(:,4)==m);
                        if size(temp_loc,1)>0 % then we have an entry
                            price_orig(i,c,t,m) = stata_orig_prices(temp_loc(1),:);
                            price_dest(i,:,c,t,m) = stata_dest_prices(temp_loc(1),:);
                        end     
                    end
                end
            end
        end
        
    % Theta
        theta = NaN(N_orig,C); % origin-commodity land holding distribution
        parfor i=1:N_orig
            for c=1:C
                temp_loc = find(stata_id(:,1)==i & stata_id(:,2)==c);
                if size(temp_loc,1)>0 % then we have an entry
                    theta(i,c) = stata_theta(temp_loc(1),:);
                end
            end
        end
        
    % Production
        production = NaN(N_orig,C,T,M); % origin-commodity-year-month production
        parfor i=1:N_orig
            for c=1:C
                for t=1:T
                    for m=1:M
                        temp_loc = find(stata_id(:,1)==i & stata_id(:,2)==c & stata_id(:,3)==t & stata_id(:,4)==m);
                        if size(temp_loc,1)>0 % then we have an entry
                            production(i,c,t,m) = stata_production(temp_loc(1),:);
                        end
                    end
                end
            end
        end        
        
        A = NaN(N_orig,C,T,M); % this is the mass of farms producing in a given month (simply using the Pareto distribution)
        parfor i=1:N_orig
            for c=1:C
                A(i,c,:,:) = production(i,c,:,:)*((theta(i,c)-1)/theta(i,c));
            end
        end
        
        Q_produce_annual = nansum(production,4); % quantity produced over entire year
        
    % Trade shares
        share_export = NaN(N_orig,N_dest,C,T); % origin-destination-commodity-year export shares
        parfor i=1:N_orig
            for c=1:C
                for t=1:T
                    share_export(i,:,c,t) = Q_trade(i,:,c,t) / nansum(Q_trade(i,:,c,t)); 
                end
            end
        end

    % Fraction of production exported
        frac_export = NaN(N_orig,C,T); % origin-commodity-year fraction of productoin exported
        frac_export = squeeze(nansum(Q_trade,2)) ./ Q_produce_annual;
        frac_export(frac_export>1) = 1; % at most, can export 100% of production (if I observe exporting more, assume it's coming from entrepot and should be ignored)
    
%% Estimating the transportation costs

    % we do this is stata to make use of the IV probit (instrumenting for
    % price ratios with rainfall in origin and destination)

    % loading the estimates (see structest_monthly_082213.do)
        stata_tauest = load('tauest_082213.dat');
        stata_sigma = load('sigmaest_082213.dat'); 
        
    % converting to Matlab format
        tau_est_incomplete = NaN(N_orig,N_dest,C);
        parfor i=1:N_orig
            for c=1:C
                temp_loc = find(stata_id(:,1)==i & stata_id(:,2)==c);
                if size(temp_loc,1)>0 % then we have an entry
                    tau_est_incomplete(i,:,c) = stata_tauest(temp_loc(1),:);
                end
            end
        end
    
%% Aside: Estimating transportation costs as if there were complete information
    sigma_est = stata_sigma; % this is the estimated sigma for the complete information case
    disp 'estimating transportation costs as if there were complete information'
    tic
    tau_est_complete = NaN(N_orig,N_dest,C); % estimated tau
    tao_max = 10;

    ML = NaN(N_orig,N_dest,C);
    parfor i=1:N_orig
        for j=1:N_dest
            for c=1:C
                price_ratio = max(squeeze(price_dest(i,j,c,:,:)) ./ squeeze(price_orig(i,c,:,:)),[],2); % maximum price ratio within a given year across months
                temp_Q = squeeze(Q_trade(i,j,c,:));
                temp_Q(isnan(price_ratio)==1)=[]; % getting rid of missing values
                anytrade = temp_Q>0; % did trade occur?
                price_ratio(isnan(price_ratio)==1)=[]; % getting rid of missing values
                if size(price_ratio,1)>0
                    if max(anytrade)==1 && min(anytrade)==0
                        [ML(i,j,c), tau_est_complete(i,j,c)] = tao_complete_071011(sigma_est,price_ratio,anytrade,zeros(size(price_ratio)),tao_max);
                    elseif min(anytrade)==1
                        tau_est_complete(i,j,c)=-99; % always trade
                    elseif max(anytrade)==0
                        tau_est_complete(i,j,c)=99; % never trade
                    end
                end
            end
        end
    end
    toc

%% Estimating the search probabilities

    V = 100; % number of simulations for simulated method of moments

    % if trade always occurs, set tau equal to one; if it never occurs,
    % set to 50
        tau_est_incomplete(tau_est_incomplete==-99) = 1;
        tau_est_incomplete(tau_est_incomplete==999) = 50;
        tau_est_incomplete(tau_est_incomplete==99) = 50;
        

    % if estimated tau is less than 1, make it equal to 1 (theoretical
    % minimum)
        tau_est_incomplete(tau_est_incomplete<1) = 1;

    % First, drawing unobserved components of the transportation costs that
    % are consistent with the observed trade flows
    
        disp('Drawing unobserved error term for simulation')
        tic

        tau_simul = NaN(N_orig,N_dest,C,T,V); % origin-destination-commodity-year-simulation tau (that must be consistent with the observed trade flows)
        
        
        parfor i=1:N_orig
            for j=1:N_dest
                for c=1:C
                    for t=1:T
                        
                        temp_thresh = log(max((squeeze(price_dest(i,j,c,t,:))/tau_est_incomplete(i,j,c)) ./ squeeze(price_orig(i,c,t,:)))); % here's the threshold
                        
                        if isfinite(temp_thresh)==1 % if we have data
                        
                            if Q_trade(i,j,c,t)>0
                                temp_draw = truncnormrnd(V,0,sigma_est,-inf,temp_thresh); % if exporting, unobserved variable must be sufficiently small
                            elseif Q_trade(i,j,c,t)==0
                                temp_draw = truncnormrnd(V,0,sigma_est,temp_thresh,inf); % if not exporting, unobserved variable must be sufficiently large   
                            end        
                            tau_simul(i,j,c,t,:) = exp(log(tau_est_incomplete(i,j,c)) + temp_draw); % here is the tau we will be using to simulate the data
                            
                        end
                    end
                end
            end
        end
        
        toc
        
    % Second, for each of these draws, we find the s_ijct that is most
    % consistent with the annual fraction of exports going to each destination
    
        s_simul = NaN(N_orig,N_dest,C,T,V); % origin-destination-commodity-simulation s (to be estimated for each s)
        f_simul = NaN(N_orig,C,T,V); % origin-commodity-year-simulation f
        
        tic
        disp 'Estimating search probabilities';
        parfor i=1:N_orig
            for c=1:C
                for t=1:T
                    if nansum(Q_trade(i,:,c,t))>0 % so you export somewhere
                        for v=1:V
                            temp_Qyr = Q_trade(i,:,c,t);
                            temp_nomiss = isfinite(temp_Qyr); % only looking at destinations with observed prices
                            temp_N = sum(isfinite(temp_Qyr)); % number of destinations with observed prices
                            temp_s_old = (1/temp_N)*ones(1,temp_N); % initial guess
                            temp_f_old = 1; % initial guess
                            tol = 0.001;
                            diff = 1;
                            iter = 0;
                            max_iter = 100;
                            while diff>tol & iter<=max_iter
                                iter = iter + 1;
                                temp_Q = NaN(temp_N,M); % destination-month trade flows
                                for m=1:M
                                    if isfinite(price_orig(i,c,t,m))==1
                                        p_dest = (price_dest(i,:,c,t,m)./tau_simul(i,:,c,t,v))'; % destination prices from a particular origin
                                        p_dest = p_dest(temp_nomiss); % only looking at places wiht positive annual trade flows
                                        temp_nomiss_month = isfinite(p_dest); % here are places with positive annual trade flows and observed monthly prices
                                        p_dest = p_dest(temp_nomiss_month); % only looking at destinations with observed prices in that month
                                        p_orig = price_orig(i,c,t,m);
                                        temp_s_old_month = temp_s_old(temp_nomiss_month); % here are the search probabilities of places wiht observed monthly prices
                                        if max(p_dest)>p_orig
                                            temp_Q_month = A(i,c,t,m)*theta(i,c)*(temp_f_old^(1-theta(i,c)))*temp_s_old_month'.*Kfun_integ_082313(p_orig,p_dest,temp_s_old_month',theta(i,c)); % quantity exported to each destination
                                            temp_Q2 = NaN(temp_N,1);
                                            temp_Q2(temp_nomiss_month) = temp_Q_month;
                                            temp_Q(:,m) = temp_Q2;
                                        elseif max(p_dest)<=p_orig
                                            temp_Q(:,m) = 0;
                                        end
                                    end
                                end

                                % summing over months
                                    temp_Q_annual = nansum(temp_Q,2); % annual trade flows to each destination
                                    temp_share = NaN(1,temp_N); % fraction of exports going to each destination
                                    for j=1:temp_N
                                        temp_share(j) = temp_Q_annual(j) / nansum(temp_Q_annual); 
                                    end

                                % updating to choose values of the search
                                % probability to match observed annual trade
                                % shares
                                    temp_actual_share = share_export(i,:,c,t);
                                    temp_actual_share = temp_actual_share(temp_nomiss);
                                    temp_s = temp_s_old.*temp_actual_share./temp_share;
                                    if sum(temp_s(isfinite(temp_s)))>1
                                        temp_s(isnan(temp_s)) = 0;
                                        temp_s = temp_s / sum(temp_s);
                                    elseif sum(temp_s(isfinite(temp_s)))<1 % then other places must have been searched too 
                                        temp_s(isnan(temp_s)) = (1-sum(temp_s(isfinite(temp_s)))) / sum(isnan(temp_s)); % give every other location equal search probabilities
                                    end

                                % updating to choose the fixed cost of search
                                % to match observed fraction exported
                                    temp_frac_export_simul = nansum(temp_Q_annual) / Q_produce_annual(i,c,t); % at the guessed fixed cost, how much is exported?
                                    temp_f = temp_f_old*(temp_frac_export_simul / frac_export(i,c,t))^(1/(theta(i,c)-1)); % what the fixed cost would need to be, all else constant, to match the observed fraction exported?
                                    if temp_f == 0 
                                        keyboard
                                    end

                                % new values
                                    diff = norm(temp_s - temp_s_old) + norm(temp_f - temp_f_old);
                                    alpha_update = 0.5;
                                    temp_s_old = alpha_update*temp_s + (1-alpha_update)*temp_s_old;
                                    temp_f_old = alpha_update*temp_f + (1-alpha_update)*temp_f_old;
                            end
                            if iter==max_iter
                                disp 'Maximum iterations reached'
                            end
                                
                            % putting the estimates back into the matrix
                                temp_s2 = NaN(1,N_dest);
                                temp_s2(temp_nomiss) = temp_s;
                                temp_s2(share_export(i,:,c,t)==0)=NaN; % only getting estimates from when trade occurs
                                temp_s2(isinf(temp_s2)) = NaN; % occurs almost never due to rounding errors
                                s_simul(i,:,c,t,v) = temp_s2;
                                f_simul(i,c,t,v) = temp_f;   
                        end
                    end
                end
            end
        end
        toc

%% Exporting the estimates to Stata

    % Averaging over years and simulations
    
        % search probabilities
            s_est_year = exp(squeeze(nanmean(log(s_simul),5)));
            s_est = nanmean(s_est_year,4);
        
        % fixed costs of search
            f_est_year = exp(squeeze(nanmean(log(f_simul),4)));
            f_est = nanmean(f_est_year,3);
            
    % Exporting matrix by matrix for loading back into stata
        
        % search probabilities
            for c=1:C
                temp = s_est(:,:,c);
                filename = strcat('sest_082213_C',num2str(c),'.est');
                save(filename, 'temp','-ascii','-tabs');
            end
            
        % fixed costs of search
            save('fest_082213.est','f_est','-ascii','-tabs');
            
        % complete information estimates of tau
            for c=1:C
                temp = tau_est_complete(:,:,c);
                filename = strcat('tauest_complete_082213_C',num2str(c),'.est');
                save(filename, 'temp','-ascii','-tabs');
            end            


    save structest_monthly_082213.mat
    
    
    
    